package xcastbot.ui;

import io.github.bonigarcia.wdm.WebDriverManager;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Date;
import java.util.List;
import java.util.function.Predicate;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.table.DefaultTableModel;
import org.openqa.selenium.By;
import org.openqa.selenium.WebDriver;
import org.openqa.selenium.WebElement;
import org.openqa.selenium.chrome.ChromeDriver;
import xcraftbot.logic.ButtonNavigate;
import xcraftbot.logic.Textbox;
import xcraftbot.xcraftobjects.Plant;
import xcraftbot.xcraftobjects.PlantType;

public class PlantPanel extends JPanel{
    
    private PlantType plantType;
    private List<Plant> plants;
    private WebDriver driverPlant;
    private JLabel lblCount;
    private JLabel lblMaxCount;
    private JLabel lblLastRefreshed;
    private JTable table;
    private JScrollPane scrollPane;
    private Date lastRefreshed;
    
    public PlantPanel(PlantType plantType, String name, String passwort){
        super();
        
        WebDriverManager.chromedriver().setup();
        driverPlant = new ChromeDriver();
        loginToXCraftCrafting(driverPlant, name, passwort);
        clickButtonByValue(driverPlant, "XCast");
        
        this.plantType = plantType;
        setLayout(new GridBagLayout());
        
        JButton btnRefresh = new JButton("Refresh");
        btnRefresh.addActionListener(((e) -> {
            refreshPlants();
        }));
        add(btnRefresh, makeGridBagConstraint(1, 0, 0));
        
        loadPlants(driverPlant);
        Predicate<Plant> isMax = (p) -> {return p.isMax();};
        Predicate<Plant> isPlantType = p-> p.getType()==plantType;
        lblCount = new JLabel("Count: "+plants.size());
        lblMaxCount = new JLabel("Max-Count: "+plants.stream().filter(isMax).count());
        lblLastRefreshed = new JLabel("Last refreshed: "+lastRefreshed.toLocaleString());
        add(lblCount, makeGridBagConstraint(1, 0, 1));
        add(lblMaxCount, makeGridBagConstraint(1, 0, 2));
        add(lblLastRefreshed, makeGridBagConstraint(1, 0, 3));
        
        Object[] plantz =  plants.stream().filter(isPlantType)
                .sorted((p1, p2) -> p2.compareTo(p1))
                .toArray();
        
        Object[][] data = new Object[plantz.length][1];
        for(int i=0;i<plantz.length;i++){
            data[i][0] = plantz[i];
        }
        String[] colNames = {"Pflanze"};
        table = new JTable(data, colNames);
        table.getColumnModel().getColumn(0).setWidth(20);
        scrollPane = new JScrollPane();
        scrollPane.getViewport().add(table);
        add(scrollPane, makeGridBagConstraint(1, 1, 0,1,5));
    }
    
    private void refreshPlants(){
        clickButtonByValue(driverPlant, "warten/aktualisieren");
        loadPlants(driverPlant);
        Predicate<Plant> isMax = (p) -> {return p.isMax();};
        Predicate<Plant> isPlantType = p-> p.getType()==plantType;
        lblCount.setText("Count: "+plants.size());
        lblMaxCount.setText("Max-Count: "+plants.stream().filter(isMax).count());
        lblLastRefreshed.setText("Last refreshed: "+lastRefreshed.toLocaleString());
        
        Object[] plantz =  plants.stream().filter(isPlantType).sorted((p1, p2) -> p2.compareTo(p1)).toArray();
        Object[][] data = new Object[plantz.length][1];
        for(int i=0;i<plantz.length;i++){
            data[i][0] = plantz[i];
        }
        String[] colNames = {"Pflanze"};
        scrollPane.remove(table);
        table = new JTable(data, colNames);
        table.getColumnModel().getColumn(0).setWidth(20);
        scrollPane.getViewport().add(table);
        scrollPane.revalidate();
        scrollPane.repaint();
        revalidate();
        repaint();
    }
    
    public void loadPlants(WebDriver driver){
        plants = new ArrayList<>();
        List<WebElement> webElements = driver.findElements(By.
                xpath("//form[@id=\"j_idt117\"]//*[contains(text(), '"+plantType.getBezeichnung()+"')]"));
        
        
        //List<WebElement> webElements = driver.findElements(By.xpath("//*[contains(text(), '"+plantType.getBezeichnung()+"')]"));
        for(WebElement elem: webElements){
            String text = elem.getText();
            String klammern = elem.getText().substring(text.indexOf("(")+1, text.indexOf(")"));
            String[] parts = klammern.split("/");
            plants.add(new Plant(plantType, Integer.parseInt(parts[0]), Integer.parseInt(parts[1]), Integer.parseInt(parts[2])));
        }
        
        lastRefreshed = new Date();
    }
    
    //<editor-fold defaultstate="collapsed" desc="Utils">
    private GridBagConstraints makeGridBagConstraint(int fill, int x, int y) {
        GridBagConstraints c = new GridBagConstraints();
        c.fill = fill;
        c.gridx = x;
        c.gridy = y;
        c.insets = new Insets(3, 3, 3, 3);
        return c;
    }
    private GridBagConstraints makeGridBagConstraint(int fill, int x, int y, int gridwidth, int gridheigt) {
        GridBagConstraints c = makeGridBagConstraint(fill, x, y);
        c.gridwidth = gridwidth;
        c.gridheight = gridheigt;
        return c;
    }
    public boolean loginToXCraftCrafting(WebDriver driver, String username, String password){
        System.out.println("Logging in");
        try{
            driver.get("https://xcraftkkrieger.cx.edvschule-plattling.de/XCraft/");
            WebElement boxName = driver.findElement(By.name(Textbox.USERNAME.getNameString()));
            boxName.sendKeys(username);
            WebElement boxPassword = driver.findElement(By.name(Textbox.PASSWORD.getNameString()));
            boxPassword.sendKeys(password);
            WebElement btnLogin = driver.findElement(By.name(ButtonNavigate.LOGIN.getNameString()));
            btnLogin.click();
            WebElement elem = findWebElementByText(driver,"Rohstoffe");
            return elem!=null;
        }catch(Exception e){
            
            e.printStackTrace();
            return false;
        }
    }
    public WebElement findWebElementByText(WebDriver driver, String text){
        WebElement elem = driver.findElement(By.xpath("//*[contains(text(), '"+text+"')]"));
        return elem;
    }
    public void clickButtonByValue(WebDriver driver,String value){
        driver.findElement(By.xpath("//input[@value='"+value+"']")).click();
    }
//</editor-fold>
    
    public void removePlant(Plant toRemove){
        for (Plant plant : plants) {
            if(plant.toRealString().equals(toRemove.toRealString())){
                plants.remove(plant);
                remove(scrollPane);
                Predicate<Plant> isPlantType = p-> p.getType()==plantType;
                Object[] plantz =  plants.stream().filter(isPlantType).sorted((p1, p2) -> p2.compareTo(p1)).toArray();
                Object[][] data = new Object[plantz.length][1];
                for(int i=0;i<plantz.length;i++){
                    data[i][0] = plantz[i];
                }
                String[] colNames = {"Pflanze"};
                scrollPane.remove(table);
                table = new JTable(data, colNames);
                table.getColumnModel().getColumn(0).setWidth(20);
                scrollPane.getViewport().add(table);
                
                scrollPane.revalidate();
                scrollPane.repaint();
                add(scrollPane, makeGridBagConstraint(1, 1, 0,1,5));
                revalidate();
                repaint();
                System.out.println("re-done jtable");
                break;
            }
        }
    }
    
    public List<Plant> getPlants() {
        return plants;
    }
    
    
}
