package xcastbot.ui;

import io.github.bonigarcia.wdm.WebDriverManager;
import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.sql.Time;
import java.util.Date;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.swing.DefaultListCellRenderer;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.ListCellRenderer;
import org.openqa.selenium.By;
import org.openqa.selenium.WebDriver;
import org.openqa.selenium.WebElement;
import org.openqa.selenium.chrome.ChromeDriver;
import xcraftbot.logic.ButtonNavigate;
import xcraftbot.logic.Textbox;
import xcraftbot.xcraftobjects.Plant;
import xcraftbot.xcraftobjects.PlantType;
import xcraftbot.xcraftobjects.Recipe;

public class TopfPanel extends JPanel{
    
    private JLabel lblName;
    private JLabel lblPflanze;
    private JLabel lblZeit;
    private JCheckBox chkAutoNoa;
    private JCheckBox chkAutoOkari;
    private int slot;
    private boolean running;
    private ZuchtPanel zuchtPanel;
    private WebDriver driverTopf;
    private JComboBox comboboxOkari;
    private JComboBox comboboxNoa;
    
    public TopfPanel(int slot, ZuchtPanel zuchtPanel){
        super();
        WebDriverManager.chromedriver().setup();
        driverTopf = new ChromeDriver();
        loginToXCraftCrafting(driverTopf, zuchtPanel.getMainFrame().getName(), zuchtPanel.getMainFrame().getPassword());
        clickButtonByValue(driverTopf, "XCast");
        setBackground(Color.LIGHT_GRAY);
        this.setLayout(new GridBagLayout());
        this.slot=slot;
        this.zuchtPanel=zuchtPanel;
        setPreferredSize(new Dimension(500, 120));
        
        if(checkRunning()){
            initRunningPanel();
        }else{
            initNotRunningPanel();
        }
        
    }
    
    private boolean checkRunning(){
        System.out.println("XPath: \"//*[@id=\\\"j_idt106\\\"]/table/tbody/tr["+slot+"]/td[1]\"");
        WebElement elem = driverTopf.findElement(By.xpath("//*[@id=\"j_idt106\"]/table/tbody/tr["+slot+"]/td[1]"));
        return !elem.getText().equals("leerer Pflanzentopf");
    }
    
    public void initRunningPanel(){
        
        clickButtonByValue(driverTopf, "warten/aktualisieren");
        
        removeAll();
        revalidate();
        repaint();
        
        lblName = new JLabel("Topf "+slot);
        
        WebElement elemPlant = driverTopf.findElement(By.xpath("//*[@id=\"j_idt106\"]/table/tbody/tr["+slot+"]/td[1]/span"));
        Plant p = Plant.parsePlant(elemPlant.getText());
        lblPflanze = new JLabel("Pflanze: "+p);
        
        WebElement elemZeit = driverTopf.findElement(By.xpath("//*[@id=\"j_idt106\"]/table/tbody/tr["+slot+"]/td[2]"));
        String timeString = elemZeit.getText().substring(elemZeit.getText().indexOf(":")+2, elemZeit.getText().length()-1);
        System.out.println("Zeit: "+timeString);
        String[] werte = timeString.split(":");
        Date zeit = new Date(0, 0, 0, Integer.parseInt(werte[0]), Integer.parseInt(werte[1]), Integer.parseInt(werte[2]));
        lblZeit = new JLabel("Zeit: "+zeit.toLocaleString());
        
        Thread t = new Thread(){
            @Override
            public void run() {
                super.run(); //To change body of generated methods, choose Tools | Templates.
                while(zeit.getSeconds()>0){
                    lblZeit.setText("Zeit: "+zeit.toLocaleString());
                    long zeitLong = zeit.getTime();
                    zeit.setTime(zeitLong-1000);
                    try {
                        sleep(1000);
                    } catch (InterruptedException ex) {
                        
                    }
                }
            }
        };
        t.start();
        
        
        chkAutoNoa = new JCheckBox("Checkbox AutoNoa");
        chkAutoOkari = new JCheckBox("Checkbox AutoOkari");
        
        setLayout(new GridBagLayout());
        add(lblName,makeGridBagConstraint(1, 0, 0));
        add(lblPflanze,makeGridBagConstraint(1, 1, 1));
        add(lblZeit,makeGridBagConstraint(1, 1, 2));
        add(chkAutoNoa,makeGridBagConstraint(1, 2, 1));
        add(chkAutoOkari,makeGridBagConstraint(1, 2, 2));
        
        
        
        
        
    }
    
    public void initNotRunningPanel(){
        
        clickButtonByValue(driverTopf, "warten/aktualisieren");
        
        removeAll();
        revalidate();
        repaint();
        if(zuchtPanel.getMainFrame().getPanelOkari().getPlants().toArray().length>0){
            Plant[] plantzOkari = zuchtPanel.getMainFrame().getPanelOkari().getPlants()
                    .stream()
                    .filter(obj -> obj instanceof Plant)
                    .map(obj -> (Plant) obj)
                    .sorted((p1, p2) -> p2.compareTo(p1)).toArray(Plant[]::new);
            comboboxOkari = new JComboBox(plantzOkari);
            comboboxOkari.setPreferredSize(new Dimension(180, 30));
            comboboxOkari.setRenderer(new ListCellRenderer() {
                @Override
                public Component getListCellRendererComponent(JList list, Object value, int index, boolean isSelected, boolean cellHasFocus) {
                    Plant p = (Plant) value;
                    JLabel renderer = (JLabel) new DefaultListCellRenderer().getListCellRendererComponent(list, p.toString(), index, isSelected, cellHasFocus);
                    return renderer;
                }
            });
            add(comboboxOkari,makeGridBagConstraint(1, 0, 0));
            JButton btnStartOkari = new JButton("Start Okari");
            add(btnStartOkari,makeGridBagConstraint(1, 1, 0));
            btnStartOkari.addActionListener(((e) -> {
                Plant selectedItem = (Plant) comboboxOkari.getSelectedItem();
                WebElement elem = driverTopf.findElement(By.xpath("//*[contains(text(), '"+selectedItem.toRealString()+"')]/following-sibling::input[1]"));
                System.out.println(elem.getAttribute("value"));
                elem.click();
                initRunningPanel();
                zuchtPanel.getMainFrame().getPanelOkari().removePlant(selectedItem);
                zuchtPanel.removeAtPanels(comboboxOkari.getSelectedIndex(), PlantType.OKARI_NUSS);
            }));
        }else{
            JLabel lblNoNoa = new JLabel("0 Okari Plant");
            add(lblNoNoa,makeGridBagConstraint(1, 0, 0));
        }
        
        if(zuchtPanel.getMainFrame().getPanelNoa().getPlants().toArray().length>0){
            Plant[] plantzNoa = zuchtPanel.getMainFrame().getPanelNoa().getPlants()
                    .stream()
                    .filter(obj -> obj instanceof Plant)
                    .map(obj -> (Plant) obj)
                    .sorted((p1, p2) -> p2.compareTo(p1)).toArray(Plant[]::new);
            comboboxNoa = new JComboBox(plantzNoa);
            comboboxNoa.setPreferredSize(new Dimension(180, 30));
            comboboxNoa.setRenderer(new ListCellRenderer() {
                @Override
                public Component getListCellRendererComponent(JList list, Object value, int index, boolean isSelected, boolean cellHasFocus) {
                    Plant p = (Plant) value;
                    JLabel renderer = (JLabel) new DefaultListCellRenderer().getListCellRendererComponent(list, p.toString(), index, isSelected, cellHasFocus);
                    return renderer;
                }
            });
            add(comboboxNoa,makeGridBagConstraint(1, 0, 2));
            JButton btnStartNoa = new JButton("Start Noa");
            add(btnStartNoa,makeGridBagConstraint(1, 1, 2));
            btnStartNoa.addActionListener(((e) -> {
                Plant selectedItem = (Plant) comboboxNoa.getSelectedItem();
                WebElement elem = driverTopf.findElement(By.xpath("//*[contains(text(), '"+selectedItem.toRealString()+"')]/following-sibling::input[1]"));
                System.out.println(elem.getAttribute("value"));
                elem.click();
                initRunningPanel();
                zuchtPanel.getMainFrame().getPanelNoa().removePlant(selectedItem);
                zuchtPanel.removeAtPanels(comboboxNoa.getSelectedIndex(), PlantType.NOA_BLATT);
            }));
        }else{
            JLabel lblNoNoa = new JLabel("0 Noa Plant");
            add(lblNoNoa,makeGridBagConstraint(1, 0, 2));
        }
    }
    
    public void removeAtIndex(int index, PlantType type){
        if(!checkRunning()){
            if(type.equals(PlantType.OKARI_NUSS)){
                comboboxOkari.removeItemAt(index);
            }else if(type.equals(PlantType.NOA_BLATT)){
                comboboxNoa.removeItemAt(index-1);
            }
        }
    }
    
    
    //<editor-fold defaultstate="collapsed" desc="Utils">
    private GridBagConstraints makeGridBagConstraint(int fill, int x, int y) {
        GridBagConstraints c = new GridBagConstraints();
        c.fill = fill;
        c.gridx = x;
        c.gridy = y;
        c.insets = new Insets(3, 3, 3, 3);
        return c;
    }
    private GridBagConstraints makeGridBagConstraint(int fill, int x, int y, int gridwidth, int gridheigt) {
        GridBagConstraints c = makeGridBagConstraint(fill, x, y);
        c.gridwidth = gridwidth;
        c.gridheight = gridheigt;
        return c;
    }
    public boolean loginToXCraftCrafting(WebDriver driver, String username, String password){
        System.out.println("Logging in");
        try{
            driver.get("https://xcraftkkrieger.cx.edvschule-plattling.de/XCraft/");
            WebElement boxName = driver.findElement(By.name(Textbox.USERNAME.getNameString()));
            boxName.sendKeys(username);
            WebElement boxPassword = driver.findElement(By.name(Textbox.PASSWORD.getNameString()));
            boxPassword.sendKeys(password);
            WebElement btnLogin = driver.findElement(By.name(ButtonNavigate.LOGIN.getNameString()));
            btnLogin.click();
            WebElement elem = findWebElementByText(driver,"Rohstoffe");
            return elem!=null;
        }catch(Exception e){
            
            e.printStackTrace();
            return false;
        }
    }
    public WebElement findWebElementByText(WebDriver driver, String text){
        WebElement elem = driver.findElement(By.xpath("//*[contains(text(), '"+text+"')]"));
        return elem;
    }
    public void clickButtonByValue(WebDriver driver,String value){
        driver.findElement(By.xpath("//input[@value='"+value+"']")).click();
    }
    //</editor-fold>
    
}
