package xcraftbot.logic;

import io.github.bonigarcia.wdm.WebDriverManager;
import static java.lang.Thread.sleep;
import java.time.format.DateTimeFormatter;
import java.time.format.DateTimeFormatterBuilder;
import java.util.ArrayList;
import java.util.List;
import org.openqa.selenium.By;
import org.openqa.selenium.WebDriver;
import org.openqa.selenium.WebElement;
import org.openqa.selenium.chrome.ChromeDriver;
import org.openqa.selenium.support.ui.ExpectedConditions;
import org.openqa.selenium.support.ui.WebDriverWait;
import xcraftbot.xcraftobjects.Account;
import xcraftbot.xcraftobjects.Geldmenge;
import xcraftbot.xcraftobjects.Item;
import xcraftbot.xcraftobjects.Material;
import xcraftbot.xcraftobjects.Recipe;
import xcraftbot.xcraftobjects.XCharacter;

public class XCraftBot {

//Attribute
    public final WebDriver driver;
    private final WebDriverWait wait;
    private WebDriver materialDriver;
    private Account account;
    
//Konstruktor
    public XCraftBot(){
        WebDriverManager.chromedriver().setup();
        driver = new ChromeDriver();
        materialDriver = new ChromeDriver();
        wait = new WebDriverWait(driver, 100);
    }
    
//Login/Logout
    public boolean loginToXCraft(WebDriver driver, String username, String password){
        driver.get("https://xcraftkkrieger.cx.edvschule-plattling.de/XCraft/");

        WebElement boxName = driver.findElement(By.name(Textbox.USERNAME.getNameString()));
        boxName.sendKeys(username);
        WebElement boxPassword = driver.findElement(By.name(Textbox.PASSWORD.getNameString()));
        boxPassword.sendKeys(password);
        WebElement btnLogin = driver.findElement(By.name(ButtonNavigate.LOGIN.getNameString()));
        btnLogin.click();
        
        account=new Account();
        account.setName(username);
        account.setPassword(password);
        account.setLoggedIn(true);
        aktualisieren(driver);
        
        return true;
    }
    public boolean loginToXCraftCrafting(WebDriver driver, String username, String password){
        driver.get("https://xcraftkkrieger.cx.edvschule-plattling.de/XCraft/");
        WebElement boxName = driver.findElement(By.name(Textbox.USERNAME.getNameString()));
        boxName.sendKeys(username);
        WebElement boxPassword = driver.findElement(By.name(Textbox.PASSWORD.getNameString()));
        boxPassword.sendKeys(password);
        WebElement btnLogin = driver.findElement(By.name(ButtonNavigate.LOGIN.getNameString()));
        btnLogin.click();
        aktualisieren(driver);
        return true;
    }
    public void logoutOfXCraft(){
        WebElement btnLogin = driver.findElement(By.name(ButtonNavigate.LOGOUT.getNameString()));
        btnLogin.click();
        
        account.setLoggedIn(false);
    }
    
//Load
    public void aktualisieren(WebDriver driver){
        driver.findElement(By.xpath("//input[@value='aktualisieren']")).click();
    }
    public void loadCharacters(){
        aktualisieren(driver);
        WebElement buttonAyron = driver.findElement(By.xpath("//input[@value='Ayron Teor']"));
        String buttonName = buttonAyron.getAttribute("name");
        List<XCharacter> chars = new ArrayList<>();
        int index = 0;
        while(true){
            String newButtonName = buttonName.substring(0, 16)+index+buttonName.substring(17);
            try{
                WebElement button = driver.findElement(By.name(newButtonName));
                XCharacter xc = loadCharacter(button.getAttribute("name"));
                xc.setXpathName("//input[@value='"+xc.getName()+"']");
                System.out.println(xc.getXpathName());
                xc.setXpathBericht("//input[@value='"+xc.getName()+"']/following-sibling::input[@value='Bericht']");
                chars.add(xc);
                index++;
            }catch(Exception e){
                break;
            }
        }
        account.setXcharacters(chars);
    }
    private XCharacter loadCharacter(String buttonName){
        XCharacter xc = new XCharacter();
        WebElement element = driver.findElement(By.name(buttonName));
        String cName = element.getAttribute("value");
        xc.setName(cName);
        element.click();
        
        //Equipment einlesen
        WebElement form = driver.findElement(By.xpath("/html/body/table[2]/tbody/tr[2]/td[2]/form"));
        String formname = form.getAttribute("name");
        int itemSlots = 10;
        int tablelength = 7;
        for(int tr=1;tr<=10;tr++){
            String name = driver.findElement(By.xpath("//*[@id=\""+formname+"\"]/table/tbody/tr["+tr+"]/td["+2+"]")).getText();
            if(name.length()>0){
                String position = driver.findElement(By.xpath("//*[@id=\""+formname+"\"]/table/tbody/tr["+tr+"]/td["+1+"]")).getText();
                String qualitaet = driver.findElement(By.xpath("//*[@id=\""+formname+"\"]/table/tbody/tr["+tr+"]/td["+3+"]")).getText();
                int quali = Integer.parseInt(qualitaet.substring(0, qualitaet.length()-1));
                int angriff = Integer.parseInt(driver.findElement(By.xpath("//*[@id=\""+formname+"\"]/table/tbody/tr["+tr+"]/td["+4+"]")).getText());
                int verteidigung = Integer.parseInt(driver.findElement(By.xpath("//*[@id=\""+formname+"\"]/table/tbody/tr["+tr+"]/td["+5+"]")).getText());
                int schaden = Integer.parseInt(driver.findElement(By.xpath("//*[@id=\""+formname+"\"]/table/tbody/tr["+tr+"]/td["+6+"]")).getText());
                int ruestung = Integer.parseInt(driver.findElement(By.xpath("//*[@id=\""+formname+"\"]/table/tbody/tr["+tr+"]/td["+7+"]")).getText());
                Item x = new Item(position,name, quali, angriff, verteidigung, schaden, ruestung, new Geldmenge(-1));
                xc.getEquipment().getItems().add(x);
            }
        }
        
        //Rezepte einlesen
        WebElement tr1td1 = driver.findElement(By.xpath("/html/body/table[2]/tbody/tr[2]/td[1]/table/tbody/tr[1]/td[1]"));
        
        int trIndex = 1;
        while(true){
            try{
                Recipe rezept = new Recipe();
                String xpathName = "/html/body/table[2]/tbody/tr[2]/td[1]/table/tbody/tr["+trIndex+"]/td[1]";
                rezept.setName(driver.findElement(By.xpath(xpathName)).getText());
                rezept.setXpath("/html/body/table[2]/tbody/tr[2]/td[1]/table/tbody/tr["+trIndex+"]/td[3]");
                String time = driver.findElement(By.xpath("/html/body/table[2]/tbody/tr[2]/td[1]/table/tbody/tr["+trIndex+"]/td[3]/form/input[2]")).getAttribute("value");
                
                int mins = Integer.parseInt(time.substring(3,5));
                int hours = Integer.parseInt(time.substring(0, 2));
                
                rezept.setTimeInMin(mins + (hours*60));
                
                xc.getRecipes().add(rezept);
                trIndex++;
            }catch(Exception e){
                break;
            }
        }
        
        
        
        
        WebElement buttonBack = driver.findElement(By.name("j_idt109:j_idt110"));
        buttonBack.click();
        return xc;
    }
    public void loadMaterials(){
        aktualisieren(driver);
        account.getMaterialInventory().getHolz().clear();
        account.getMaterialInventory().getMetall().clear();
        account.getMaterialInventory().getPelz().clear();
        account.getMaterialInventory().getLeder().clear();
        String formid = driver.findElement(By.xpath("/html/body/span[1]/form")).getAttribute("name");
        
        for(int a=1;a<=4;a++){
            System.out.println("Materialtype: "+a);
            for(int b=1;b<=4;b++){
                String text = driver.findElement(By.xpath("//*[@id=\""+formid+"\"]/table/tbody/tr["+a+"]/td/table/tbody/tr["+b+"]/td")).getText();
                int count = 0;
                if(text.length()>=1){
                    String digits = text.replaceAll("[^0-9.]", "");
                    count = digits.length()<1?0:Integer.parseInt(digits);
                }
                switch(a){
                    case 1:account.getMaterialInventory().getLeder().add(new Material(count, Material.LEDER[b-1], b));break;
                    case 2:account.getMaterialInventory().getHolz().add(new Material(count, Material.HOLZ[b-1], b));break;
                    case 3:account.getMaterialInventory().getMetall().add(new Material(count, Material.METALL[b-1], b));break;
                    case 4:account.getMaterialInventory().getPelz().add(new Material(count, Material.PELZ[b-1], b));break;
                }
            }
        }
        
    }
    
  
//Buttons
    public void btn1Click(String xpathc, String xpathi, int count, int time){
        
        Thread t = new Thread() {
            public void run() {
                int i=0;
                while(i<count){
                    try {
                        
                        //wait.until(ExpectedConditions.visibilityOfElementLocated(By.xpath("//*[@id=\"j_idt39\"]/table/tbody/tr[1]/td/div/input")));
                        driver.findElement(By.xpath(xpathc)).click();
                        //wait.until(ExpectedConditions.visibilityOfElementLocated(By.xpath("//*[@id=\"j_idt61:0:craftTemplate:craftButton\"]")));
                        driver.findElement(By.xpath(xpathi)).click();
                        sleep((time*60*1000)+500);
                        aktualisieren(driver);
                        //wait.until(ExpectedConditions.visibilityOfElementLocated(By.xpath("//*[@id=\"j_idt39\"]/table/tbody/tr[1]/td/div/input[2]")));
                        driver.findElement(By.xpath("//*[@id=\"j_idt39\"]/table/tbody/tr[1]/td/div/input[2]")).click();
                        //wait.until(ExpectedConditions.visibilityOfElementLocated(By.xpath("//*[@id=\"j_idt15\"]/input[3]")));
                        driver.findElement(By.xpath("//*[@id=\"j_idt15\"]/input[3]")).click();
                        i++;
                        
                    } catch (InterruptedException ex) {

                    }
                    
                }
                this.stop();
            }
        };
        t.start();
    }
    
    public void btn2Click(XCharacter xc, Recipe r, int anzahl){
        System.out.println("Starting process");
                
        Thread t = new Thread() {
            public void run() {
                WebDriver driverNew = new ChromeDriver();
                loginToXCraftCrafting(driverNew, account.getName(), account.getPassword());
                int i=0;
                while(i<anzahl){
                    try {
                        
                        //click on Character
                        driverNew.findElement(By.xpath(xc.getXpathName())).click();
                        //click on CraftingItem
                        driverNew.findElement(By.xpath(r.getXpath())).click();
                        sleep(1000*60*r.getTimeInMin()+500);
                        aktualisieren(driverNew);
                        //click on Bericht
                        driverNew.findElement(By.xpath(xc.getXpathBericht())).click();
                        //click on backZurUebersicht
                        driverNew.findElement(By.xpath("//input[@value='zur Übersicht']")).click();
                        i++;
                        System.out.println("One cycle done");
                                
                    } catch (InterruptedException ex) {

                    }
                    
                }
                System.out.println("process done");
                driverNew.quit();
                this.stop();
            }
        };
        t.start();
    }
    
//Getter und Setter
    public Account getAccount() {
        return account;
    }
    public void setAccount(Account account) {
        this.account = account;
    }
}
