package xcraftbot.ui;

import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.List;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JPasswordField;
import javax.swing.JScrollPane;
import javax.swing.JTabbedPane;
import javax.swing.JTable;
import javax.swing.JTextField;
import xcraftbot.logic.XCraftBot;
import xcraftbot.xcraftobjects.Item;
import xcraftbot.xcraftobjects.Material;
import xcraftbot.xcraftobjects.XCharacter;
import java.awt.Color;
import java.awt.Component;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Comparator;
import java.util.logging.Level;
import java.util.logging.Logger;
import java.util.stream.Stream;
import javax.swing.DefaultListCellRenderer;
import javax.swing.JComboBox;
import javax.swing.JList;
import javax.swing.JOptionPane;
import javax.swing.JTextPane;
import javax.swing.ListCellRenderer;
import javax.swing.table.DefaultTableCellRenderer;
import javax.swing.table.TableCellRenderer;
import xcraftbot.xcraftobjects.Recipe;

public class SwingUI2 {

    // <editor-fold defaultstate="collapsed" desc=" Region: Attributes ">
    private JFrame frame;
    private JTabbedPane tabbedPane;
    private XCraftBot xbot;
    // </editor-fold>

    // <editor-fold defaultstate="collapsed" desc=" Region: Constructors and Main ">
    public SwingUI2(XCraftBot xbot) {
        this.xbot = xbot;
        
        frame = new JFrame("XCraft Bot");
        frame.setExtendedState(JFrame.MAXIMIZED_BOTH);
        frame.setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);
        frame.addWindowListener(new java.awt.event.WindowAdapter() {
            @Override
            public void windowClosing(java.awt.event.WindowEvent windowEvent) {
                if (JOptionPane.showConfirmDialog(frame,
                        "Are you sure you want to close this window?", "Close Window?",
                        JOptionPane.YES_NO_OPTION,
                        JOptionPane.QUESTION_MESSAGE) == JOptionPane.YES_OPTION) {
                    xbot.driver.close();
                    try {
                        Runtime rt = Runtime.getRuntime();
                        Process proc = rt.exec("taskkill /im chromedriver.exe /f /t");
                    } catch (IOException ex) {
                    }
                    System.exit(0);
                }
            }
        });
        
        tabbedPane = new JTabbedPane();
        tabbedPane.addTab("Login", makeLoginPanel());
        frame.getContentPane().add(tabbedPane);
        
        frame.setVisible(true);
    }
    
    public static void main(String[] args) {
        XCraftBot xbot = new XCraftBot();
        SwingUI2 ui = new SwingUI2(xbot);
    }

// </editor-fold>
    
    // <editor-fold defaultstate="collapsed" desc=" Region: Pop-Up Windows ">
    private void showRuestung(String slot) {
        JFrame f = new JFrame("Slot");
        f.setSize(800, 800);
        f.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
        f.getContentPane().setLayout(new GridBagLayout());
        
        Object[][] data = new Object[xbot.getAccount().getXcharacters().size()][3];
        List<Integer> werte = new ArrayList<>();
        String[] colNames = {"Character", "Name", "Wert"};
        
        for (int i = 0; i < xbot.getAccount().getXcharacters().size(); i++) {
            XCharacter dude = xbot.getAccount().getXcharacters().get(i);
            data[i][0] = dude.getName();
            
            for (Item it : dude.getEquipment().getItems()) {
                if (it.getPosition().equals(slot)) {
                    data[i][1] = it.getGegenstand();
                    data[i][2] = it.getRuestung();
                    werte.add(it.getRuestung());
                    break;
                } else {
                    data[i][1] = "---";
                    data[i][2] = "---";
                }
            }
        }
        
        JTable table = new JTable(data, colNames);
        table.getColumnModel().getColumn(0).setMinWidth(100);
        table.getColumnModel().getColumn(1).setMinWidth(300);
        table.getColumnModel().getColumn(0).setMinWidth(50);
        table.setPreferredSize(new Dimension(700, 700));
        table.setPreferredScrollableViewportSize(table.getPreferredSize());
        table.setFillsViewportHeight(true);
        //table.setEnabled(false);
        
        table.setDefaultRenderer(Object.class, new TableCellRenderer() {
            private DefaultTableCellRenderer DEFAULT_RENDERER = new DefaultTableCellRenderer();

            @Override
            public Component getTableCellRendererComponent(JTable table, Object value, boolean isSelected, boolean hasFocus, int row, int column) {
                Component c = DEFAULT_RENDERER.getTableCellRendererComponent(table, value, isSelected, hasFocus, row, column);
                if (column == 2) {
                    if (value instanceof String) {
                        String wert = (String) value;
                        if (wert.equals("---")) {
                            c.setBackground(Color.GRAY);
                        } else {
                            c.setBackground(Color.WHITE);
                        }
                    } else {
                        int wert = (int) value;
                        if (wert == 0) {
                            c.setBackground(Color.GRAY);
                        } else {
                            Integer maxNumber = werte.stream().max(Comparator.comparing(Integer::valueOf)).get();
                            Integer minNumber = werte.stream().min(Comparator.comparing(Integer::valueOf)).get();
                            if (wert == maxNumber) {
                                c.setBackground(Color.GREEN);
                            } else if (wert > minNumber) {
                                c.setBackground(Color.YELLOW);
                            } else {
                                c.setBackground(Color.RED);
                            }
                        }
                    }
                } else {
                    c.setBackground(Color.WHITE);
                }
                
                return c;
            }
        });
        
        JScrollPane scrollPane = new JScrollPane(table);
        f.getContentPane().add(scrollPane);
        f.setVisible(true);
    }

    private void showBewaffnung(String slot) {
        JFrame f = new JFrame("Slot");
        f.setSize(800, 800);
        f.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
        f.getContentPane().setLayout(new GridBagLayout());
        
        Object[][] data = new Object[xbot.getAccount().getXcharacters().size()][5];
        String[] colNames = {"Character", "Name", "Angriff", "Verteidigung", "Schaden"};
        
        for (int i = 0; i < xbot.getAccount().getXcharacters().size(); i++) {
            XCharacter dude = xbot.getAccount().getXcharacters().get(i);
            data[i][0] = dude.getName();
            
            for (Item it : dude.getEquipment().getItems()) {
                if (it.getPosition().equals(slot)) {
                    System.out.println(it.getPosition());
                    data[i][1] = it.getGegenstand();
                    data[i][2] = it.getAngriff();
                    data[i][3] = it.getVerteidigung();
                    data[i][4] = it.getSchaden();
                    break;
                } else {
                    data[i][1] = "---";
                    data[i][2] = "---";
                }
            }
        }
        
        JTable table = new JTable(data, colNames);
        table.getColumnModel().getColumn(0).setMinWidth(100);
        table.getColumnModel().getColumn(1).setMinWidth(300);
        table.getColumnModel().getColumn(0).setMinWidth(50);
        table.setPreferredSize(new Dimension(700, 700));
        table.setPreferredScrollableViewportSize(table.getPreferredSize());
        table.setFillsViewportHeight(true);
        //table.setEnabled(false);
        
        JScrollPane scrollPane = new JScrollPane(table);
        f.getContentPane().add(scrollPane);
        f.setVisible(true);
    }

    private void showBewaffnungGesamt() {
        JFrame f = new JFrame("Slot");
        f.setSize(800, 800);
        f.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
        f.getContentPane().setLayout(new GridBagLayout());
        
        Object[][] data = new Object[xbot.getAccount().getXcharacters().size()][5];
        String[] colNames = {"Character", "Name", "Angriff", "Verteidigung", "Schaden"};
        
        for (int i = 0; i < xbot.getAccount().getXcharacters().size(); i++) {
            XCharacter dude = xbot.getAccount().getXcharacters().get(i);
            data[i][0] = dude.getName();
            data[i][1] = "";
            data[i][2] = 0;
            data[i][3] = 0;
            data[i][4] = 0;
            
            for (Item it : dude.getEquipment().getItems()) {
                if (it.getPosition().equals("Zweihandwaffe") || it.getPosition().equals("Waffenhand")) {
                    data[i][1] = it.getGegenstand().split("\\(")[0];
                    data[i][2] = (int) data[i][2] + it.getAngriff();
                    data[i][3] = (int) data[i][3] + it.getVerteidigung();
                    data[i][4] = (int) data[i][4] + it.getSchaden();
                } else if (it.getPosition().equals("Schildhand")) {
                    data[i][1] += " + " + it.getGegenstand().split("\\(")[0];
                }
            }
        }
        
        JTable table = new JTable(data, colNames);
        table.getColumnModel().getColumn(0).setMinWidth(100);
        table.getColumnModel().getColumn(1).setMinWidth(300);
        table.getColumnModel().getColumn(2).setMinWidth(50);
        table.getColumnModel().getColumn(3).setMinWidth(50);
        table.getColumnModel().getColumn(4).setMinWidth(50);
        table.setPreferredSize(new Dimension(700, 700));
        table.setPreferredScrollableViewportSize(table.getPreferredSize());
        table.setFillsViewportHeight(true);
        
        JScrollPane scrollPane = new JScrollPane(table);
        f.getContentPane().add(scrollPane);
        f.setVisible(true);
    }

// </editor-fold>
    
    // <editor-fold defaultstate="collapsed" desc=" Region: Make Panels ">
    private JPanel makeLoginPanel() {
        JPanel p = new JPanel();
        p.setLayout(new GridBagLayout());
        
        JLabel lblName = new JLabel("Name:");
        p.add(lblName, makeGridBagConstraint(GridBagConstraints.CENTER, 0, 0));
        JLabel lblPassword = new JLabel("Password:");
        p.add(lblPassword, makeGridBagConstraint(GridBagConstraints.CENTER, 0, 1));
        
        JTextField txtName = new JTextField();
        txtName.setPreferredSize(new Dimension(150, 30));
        p.add(txtName, makeGridBagConstraint(GridBagConstraints.CENTER, 1, 0));
        JPasswordField txtPassword = new JPasswordField();
        txtPassword.setPreferredSize(new Dimension(150, 30));
        p.add(txtPassword, makeGridBagConstraint(GridBagConstraints.CENTER, 1, 1));
        
        JButton btnLogin = new JButton("Login");
        btnLogin.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                login(txtName.getText(), txtPassword.getText());
            }
        });
        p.add(btnLogin, makeGridBagConstraint(GridBagConstraints.CENTER, 0, 2, 2, 1));
        
        return p;
    }

    private JPanel makeOverviewPanel() {
        JPanel p = new JPanel();
        p.setLayout(new GridBagLayout());
        
        JLabel lblRuestungen = new JLabel("Ruestungen");
        p.add(lblRuestungen, makeGridBagConstraint(GridBagConstraints.CENTER, 0, 0));
        String[] ruestungsbuttons = {"Kopf", "Rücken", "Körper", "Arme", "Hände", "Beine", "Füsse"};
        for (int i = 0; i < ruestungsbuttons.length; i++) {
            JButton btn = new JButton(ruestungsbuttons[i]);
            btn.addActionListener((ActionEvent e) -> {
                showRuestung(btn.getText());
            });
            p.add(btn, makeGridBagConstraint(GridBagConstraints.CENTER, 0, i + 1));
        }
        JLabel lblBewaffnung = new JLabel("Bewaffnung");
        p.add(lblBewaffnung, makeGridBagConstraint(GridBagConstraints.CENTER, 0, 1 + ruestungsbuttons.length));
        String[] waffenbuttons = {"Zweihandwaffe", "Waffenhand", "Schildhand"};
        for (int i = 0; i < waffenbuttons.length; i++) {
            JButton btn = new JButton(waffenbuttons[i]);
            btn.addActionListener((ActionEvent e) -> {
                showBewaffnung(btn.getText());
            });
            p.add(btn, makeGridBagConstraint(GridBagConstraints.CENTER, 0, i + ruestungsbuttons.length + 2));
        }
        JButton btn = new JButton("Gesamtbewaffnung");
        btn.addActionListener((ActionEvent e) -> {
            showBewaffnungGesamt();
        });
        p.add(btn, makeGridBagConstraint(GridBagConstraints.CENTER, 0, waffenbuttons.length + ruestungsbuttons.length + 2));
        
        return p;
    }

    private JPanel makeMaterialPanel() {
        JPanel p = new JPanel();
        p.setLayout(new GridBagLayout());
        
        JLabel lblHolz = new JLabel("Holz");
        p.add(lblHolz, makeGridBagConstraint(GridBagConstraints.NONE, 0, 0));
        JTable table = makeMaterialTable(xbot.getAccount().getMaterialInventory().getHolz());
        JScrollPane scrollPane = new JScrollPane(table);
        scrollPane.setPreferredSize(new Dimension(350, 90));
        p.add(scrollPane, makeGridBagConstraint(GridBagConstraints.NONE, 0, 1));
        
        JLabel lblMetall = new JLabel("Metall");
        p.add(lblMetall, makeGridBagConstraint(GridBagConstraints.NONE, 1, 0));
        JTable table2 = makeMaterialTable(xbot.getAccount().getMaterialInventory().getMetall());
        JScrollPane scrollPane2 = new JScrollPane(table2);
        scrollPane2.setPreferredSize(new Dimension(350, 90));
        p.add(scrollPane2, makeGridBagConstraint(GridBagConstraints.NONE, 1, 1));
        
        JLabel lblLeder = new JLabel("Leder");
        p.add(lblLeder, makeGridBagConstraint(GridBagConstraints.NONE, 0, 2));
        JTable table3 = makeMaterialTable(xbot.getAccount().getMaterialInventory().getLeder());
        JScrollPane scrollPane3 = new JScrollPane(table3);
        scrollPane3.setPreferredSize(new Dimension(350, 90));
        p.add(scrollPane3, makeGridBagConstraint(GridBagConstraints.NONE, 0, 3));
        
        JLabel lblPelz = new JLabel("Pelz");
        p.add(lblPelz, makeGridBagConstraint(GridBagConstraints.NONE, 1, 2));
        JTable table4 = makeMaterialTable(xbot.getAccount().getMaterialInventory().getPelz());
        JScrollPane scrollPane4 = new JScrollPane(table4);
        scrollPane4.setPreferredSize(new Dimension(350, 90));
        p.add(scrollPane4, makeGridBagConstraint(GridBagConstraints.NONE, 1, 3));
        
        JButton btn = new JButton("Reload");
        btn.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                tabbedPane.remove(1);
                xbot.loadMaterials();
                tabbedPane.add("Material", makeMaterialPanel());
                tabbedPane.setSelectedIndex(1);
            }
        });
        
        GridBagConstraints gbc = makeGridBagConstraint(GridBagConstraints.NONE, 1, 4);
        gbc.gridwidth = 2;
        p.add(btn, gbc);
        
        return p;
    }

    private JTable makeMaterialTable(List<Material> mats) {
        Object[][] data = new Object[mats.size()][2];
        String[] colNames = {"Name", "Anzahl"};
        
        for (int i = 0; i < mats.size(); i++) {
            data[i][0] = mats.get(i).getName();
            data[i][1] = mats.get(i).getAmount();
        }
        
        JTable table = new JTable(data, colNames);
        
        table.setDefaultRenderer(Object.class, new TableCellRenderer() {
            private DefaultTableCellRenderer DEFAULT_RENDERER = new DefaultTableCellRenderer();

            @Override
            public Component getTableCellRendererComponent(JTable table, Object value, boolean isSelected, boolean hasFocus, int row, int column) {
                Component c = DEFAULT_RENDERER.getTableCellRendererComponent(table, value, isSelected, hasFocus, row, column);
                if (isSelected) {
                    c.setBackground(Color.YELLOW);
                } else {
                    if (row % 2 == 0) {
                        c.setBackground(Color.WHITE);
                        
                    } else {
                        c.setBackground(Color.LIGHT_GRAY);
                    }                    
                }
                return c;
            }
        });
        
        return table;
    }

    private JPanel makeCraftingPanel() {
        JPanel p = new JPanel();
        p.setLayout(new GridBagLayout());
        
        JComboBox comboboxCharacter = new JComboBox(xbot.getAccount().getXcharacters().toArray());
        comboboxCharacter.setPreferredSize(new Dimension(100, 30));
        comboboxCharacter.setRenderer(new ListCellRenderer() {
            @Override
            public Component getListCellRendererComponent(JList list, Object value, int index, boolean isSelected, boolean cellHasFocus) {
                XCharacter xc = (XCharacter) value;
                JLabel renderer = (JLabel) new DefaultListCellRenderer().getListCellRendererComponent(list, xc.getName(), index, isSelected, cellHasFocus);
                return renderer;
            }
        });
        comboboxCharacter.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                for (Component c : p.getComponents()) {
                    if (c instanceof JComboBox) {
                        JComboBox jcb = (JComboBox) c;
                        if (jcb.getSelectedItem() instanceof Recipe) {
                            p.remove(c);
                            p.repaint();
                            p.getParent().repaint();
                        }
                    }
                }
                
                XCharacter xc = (XCharacter) comboboxCharacter.getSelectedItem();
                JComboBox comboboxRecipes = new JComboBox(xc.getRecipes().toArray());
                comboboxRecipes.setPreferredSize(new Dimension(180, 30));
                comboboxRecipes.setRenderer(new ListCellRenderer() {
                    @Override
                    public Component getListCellRendererComponent(JList list, Object value, int index, boolean isSelected, boolean cellHasFocus) {
                        Recipe recipe = (Recipe) value;
                        JLabel renderer = (JLabel) new DefaultListCellRenderer().getListCellRendererComponent(list, recipe.getName(), index, isSelected, cellHasFocus);
                        return renderer;
                    }
                });
                p.add(comboboxRecipes, makeGridBagConstraint(GridBagConstraints.CENTER, 1, 0));
                p.repaint();
                p.getParent().repaint();
                
            }
        });
        p.add(comboboxCharacter, makeGridBagConstraint(GridBagConstraints.CENTER, 0, 0));
        
        XCharacter xc = (XCharacter) comboboxCharacter.getSelectedItem();
        JComboBox comboboxRecipes = new JComboBox(xc.getRecipes().toArray());
        comboboxRecipes.setPreferredSize(new Dimension(180, 30));
        comboboxRecipes.setRenderer(new ListCellRenderer() {
            @Override
            public Component getListCellRendererComponent(JList list, Object value, int index, boolean isSelected, boolean cellHasFocus) {
                Recipe recipe = (Recipe) value;
                JLabel renderer = (JLabel) new DefaultListCellRenderer().getListCellRendererComponent(list, recipe.getName(), index, isSelected, cellHasFocus);
                return renderer;
            }
        });
        p.add(comboboxRecipes, makeGridBagConstraint(GridBagConstraints.CENTER, 1, 0));
        p.repaint();
        
        JLabel lbl4 = new JLabel("Anzahl:");
        p.add(lbl4, makeGridBagConstraint(GridBagConstraints.CENTER, 2, 0));
        JTextField txtCountNew = new JTextField();
        txtCountNew.setPreferredSize(new Dimension(100, 30));
        p.add(txtCountNew, makeGridBagConstraint(GridBagConstraints.CENTER, 3, 0));
        
        JButton btn2 = new JButton("Click");
        btn2.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                XCharacter xc = (XCharacter) comboboxCharacter.getSelectedItem();
                System.out.println("Character: " + xc.getName());
                int anzahl = Integer.parseInt(txtCountNew.getText());
                System.out.println("Anzahl: " + anzahl);                
                for (Component c : p.getComponents()) {
                    if (c instanceof JComboBox) {
                        JComboBox jcb = (JComboBox) c;
                        if (jcb.getSelectedItem() instanceof Recipe) {
                            Recipe r = (Recipe) jcb.getSelectedItem();
                            System.out.println("Recipe: " + r.getName() + " in " + r.getTimeInMin() + "mins");
                            xbot.btn2Click(xc, r, anzahl);
                            break;
                        }
                    }
                }
            }
        });
        p.add(btn2, makeGridBagConstraint(GridBagConstraints.CENTER, 4, 0));
        
        JButton btnMarketsnipe = new JButton("Marketplace Sniper");
        p.add(btnMarketsnipe,makeGridBagConstraint(GridBagConstraints.CENTER, 0, 1));
        
        return p;
    }
    
    private JPanel makeFeaturesPanel() {
        JPanel p = new JPanel();
        p.setLayout(new GridBagLayout());
        
        JButton btnMarketplace = new JButton("Marketplace Sniper");
        btnMarketplace.addActionListener((e) -> {
            MarketplaceUI marketplaceUI = new MarketplaceUI();
        });
        p.add(btnMarketplace, makeGridBagConstraint(GridBagConstraints.CENTER, 0, 0));
        
        JButton btnXCastFarmPlant = new JButton("XCast Plant Farm");
        p.add(btnXCastFarmPlant, makeGridBagConstraint(GridBagConstraints.CENTER, 0, 1));
        
        JButton btnXCastHealthFarm = new JButton("XCast Health Farm");
        p.add(btnXCastHealthFarm, makeGridBagConstraint(GridBagConstraints.CENTER, 0, 2));
        
        return p;
    }

    
// </editor-fold>
    
    // <editor-fold defaultstate="collapsed" desc=" Region: Random Stuff ">
    private void login(String username, String password) {
        xbot.loginToXCraft(xbot.driver, username, password);
        tabbedPane.remove(0);
        tabbedPane.add("Overview", makeOverviewPanel());
        xbot.loadCharacters();
        xbot.loadMaterials();
        tabbedPane.add("Materials", makeMaterialPanel());
        tabbedPane.add("Crafting", makeCraftingPanel());
        
        for (XCharacter xc : xbot.getAccount().getXcharacters()) {
            System.out.println(Arrays.toString(xc.getEquipment().getItems().toArray()));
        }
    }

    private GridBagConstraints makeGridBagConstraint(int fill, int x, int y) {
        GridBagConstraints c = new GridBagConstraints();
        c.fill = fill;
        c.gridx = x;
        c.gridy = y;
        c.insets = new Insets(3, 3, 3, 3);
        return c;
    }

    private GridBagConstraints makeGridBagConstraint(int fill, int x, int y, int gridwidth, int gridheigt) {
        GridBagConstraints c = makeGridBagConstraint(fill, x, y);
        c.gridwidth = gridwidth;
        c.gridheight = gridheigt;
        return c;
    }

// </editor-fold>
}
