package xcraftbot.ui;

import io.github.bonigarcia.wdm.WebDriverManager;
import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.List;
import java.util.function.Predicate;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JPasswordField;
import javax.swing.JScrollPane;
import javax.swing.JTabbedPane;
import javax.swing.JTable;
import javax.swing.JTextField;
import javax.swing.table.DefaultTableCellRenderer;
import javax.swing.table.TableCellRenderer;
import net.bytebuddy.agent.builder.AgentBuilder;
import org.openqa.selenium.By;
import org.openqa.selenium.JavascriptExecutor;
import org.openqa.selenium.WebDriver;
import org.openqa.selenium.WebElement;
import org.openqa.selenium.chrome.ChromeDriver;
import xcraftbot.logic.ButtonNavigate;
import xcraftbot.logic.Textbox;
import xcraftbot.xcraftobjects.Plant;
import xcraftbot.xcraftobjects.PlantType;

public class XCastOkariFarm {
    
    public XCastOkariFarm(){
        WebDriverManager.chromedriver().setup();
        WebDriver driver = new ChromeDriver();
        
        JFrame frame = new JFrame("Marketplace");
        frame.setSize(300, 300);
        frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        JPanel panel = new JPanel();
        panel.setLayout(null);
        
        JLabel lblName = new JLabel("Name:");
        JTextField txtName = new JTextField();
        JLabel lblPassword = new JLabel("Password:");
        JPasswordField txtPassword = new JPasswordField();
        
        JButton btnBotLogin = new JButton("Login");
        
        lblName.setBounds(10, 10, 100, 30);
        txtName.setBounds(120, 10, 100, 30);
        lblPassword.setBounds(10, 50, 100, 30);
        txtPassword.setBounds(120,50,100,30);
        
        btnBotLogin.setBounds(10, 90, 210, 30);
        
        panel.add(lblName);
        panel.add(txtName);
        panel.add(lblPassword);
        panel.add(txtPassword);
        
        panel.add(btnBotLogin);
        
        btnBotLogin.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                boolean success=true;
                try{
                    loginToXCraftCrafting(driver, txtName.getText(), txtPassword.getText());
                    panel.removeAll();
                    panel.repaint();
                    frame.repaint();
                    init(frame, txtName.getText(), driver);
                }catch (Exception ex){
                    System.out.println("Login failed!");
                    System.out.println(ex);
                    success=false;
                }
            }
        });
        
        frame.add(panel);
        frame.setVisible(true);
        frame.setVisible(true);
    }
    
    public void init(JFrame frameOld, String name, WebDriver driver){
        frameOld.dispose();
        
        JFrame frame = new JFrame();
        frame.setSize(400, 400);
        frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        
        JTabbedPane tabbedPane = new JTabbedPane();
        JPanel panelOverview = new JPanel();
        tabbedPane.addTab("Overview", panelOverview);
        frame.add(tabbedPane);
        
        JLabel lblName = new JLabel(name);
        lblName.setBounds(10, 10, 100, 30);
        panelOverview.add(lblName);
        
        clickButtonByValue(driver, "XCast");
        
        List<Plant> plants = loadPlants(driver);
        Predicate<Plant> predicateOkari = p-> p.getType()==PlantType.OKARI_NUSS;
        
        Object[] okaris =  plants.stream().filter(predicateOkari)
                .sorted((p1, p2) -> p2.compareTo(p1))
                .toArray();
        JLabel lblCount = new JLabel("Okari-Nuesse: "+okaris.length);
        lblCount.setBounds(15, 50, 150, 30);
        panelOverview.add(lblCount);
        
        JPanel panelOkari = new JPanel();
        Object[][] data = new Object[okaris.length][1];
        for(int i=0;i<okaris.length;i++){
            data[i][0] = okaris[i];
        }
        String[] colNames = {"Okari-Nuss"};
        JTable tableOkari= new JTable(data, colNames);
        tableOkari.setDefaultRenderer(Object.class, new TableCellRenderer() {
            private DefaultTableCellRenderer DEFAULT_RENDERER = new DefaultTableCellRenderer();

            @Override
            public Component getTableCellRendererComponent(JTable table, Object value, boolean isSelected, boolean hasFocus, int row, int column) {
                Component c = DEFAULT_RENDERER.getTableCellRendererComponent(table, value, isSelected, hasFocus, row, column);
                if (isSelected) {
                    c.setBackground(new Color(c.getBackground().getRGB()+40));
                } else {
                    Plant plant = (Plant) value;
                    if(plant.isMax()){
                        c.setBackground(Color.GREEN);
                    }else{
                        c.setBackground(Color.LIGHT_GRAY);
                    }
                }
                return c;
            }
        });
        tableOkari.getColumnModel().getColumn(0).setPreferredWidth(200);
        
        
        JScrollPane scrollPaneOkari = new JScrollPane(tableOkari);
        panelOkari.add(scrollPaneOkari);
        tabbedPane.add(panelOkari,"Okari-Nuesse");
        
        frame.setVisible(true);
        
    }
    
    public static void main(String[] args) {
        XCastOkariFarm ui = new XCastOkariFarm();
    }
    
    public boolean loginToXCraftCrafting(WebDriver driver, String username, String password){
        driver.get("https://xcraftkkrieger.cx.edvschule-plattling.de/XCraft/");
        WebElement boxName = driver.findElement(By.name(Textbox.USERNAME.getNameString()));
        boxName.sendKeys(username);
        WebElement boxPassword = driver.findElement(By.name(Textbox.PASSWORD.getNameString()));
        boxPassword.sendKeys(password);
        WebElement btnLogin = driver.findElement(By.name(ButtonNavigate.LOGIN.getNameString()));
        btnLogin.click();
        return true;
    }
    
    public void clickButtonByValue(WebDriver driver,String value){
        driver.findElement(By.xpath("//input[@value='"+value+"']")).click();
    }
    
    public int countByText(WebDriver driver, String text){
        List<WebElement> webElements = driver.findElements(By.xpath("//*[contains(text(), '"+text+"')]"));
        return webElements.size();
    }
    
    public List<Plant> loadPlants(WebDriver driver){
        JFrame frame = new JFrame("Loading Plants!");
        frame.setSize(200, 100);
        frame.setLocation(500, 500);
        JLabel lblStatus = new JLabel("Loaded: 0");
        frame.getContentPane().add(lblStatus);
        frame.setVisible(true);
        List<Plant> plants = new ArrayList<>();
        int count = 0;
        for(PlantType planttype:PlantType.values()){
            System.out.println("Start findElements");
            List<WebElement> webElements = driver.findElements(By.xpath("//*[contains(text(), '"+planttype.getBezeichnung()+"')]"));
            System.out.println("End findElements");
            for(WebElement elem: webElements){
                String text = elem.getText();
                String klammern = elem.getText().substring(text.indexOf("(")+1, text.indexOf(")"));
                String[] parts = klammern.split("/");
                plants.add(new Plant(planttype, Integer.parseInt(parts[0]), Integer.parseInt(parts[1]), Integer.parseInt(parts[2])));
                count++;
                lblStatus.setText("Loaded: "+count);
                System.out.print("adding");
            }
            System.out.println("");
                    
        }
        System.out.println("Done loading");
        return plants;
    }
    
}
